using SessionZero.Models;
using System; // Added for ArgumentException and StringComparison
using System.Collections.Generic;
using System.Linq;
using System.Text.Json;
using System.Threading.Tasks;

namespace SessionZero.Services;

public class SzfStorageService : ISzfStorageService
{
    private readonly ILocalStorageService _localStorageService;
    private const string SzfFileKeyPrefix = "szf_file_"; // Prefix for .szf file keys in LocalStorage

    public SzfStorageService(ILocalStorageService localStorageService)
    {
        _localStorageService = localStorageService;
    }

    private string GetKeyForSzfFile(string guid) => $"{SzfFileKeyPrefix}{guid}";

    public async Task SaveSzfFileAsync(SzfFile szfFile)
    {
        if (string.IsNullOrEmpty(szfFile.Guid))
        {
            throw new ArgumentException("GUID cannot be empty for an SzfFile.", nameof(szfFile.Guid));
        }

        var key = GetKeyForSzfFile(szfFile.Guid);
        var jsonContent = JsonSerializer.Serialize(szfFile);
        await _localStorageService.SetItemAsync(key, jsonContent);
    }

    public async Task<SzfFile?> GetSzfFileByGuidAsync(string guid)
    {
        var key = GetKeyForSzfFile(guid);
        var jsonContent = await _localStorageService.GetItemAsync(key);
        if (jsonContent == null)
        {
            return null;
        }

        return JsonSerializer.Deserialize<SzfFile>(jsonContent);
    }

    public async Task<IEnumerable<SzfFile>> GetAllSzfFilesAsync()
    {
        var allSzfFiles = new List<SzfFile>();
        var allKeys = await _localStorageService.GetAllKeysAsync(); // Utilizes the new method

        foreach (var key in allKeys)
        {
            if (key.StartsWith(SzfFileKeyPrefix))
            {
                var jsonContent = await _localStorageService.GetItemAsync(key);
                if (jsonContent != null)
                {
                    var szfFile = JsonSerializer.Deserialize<SzfFile>(jsonContent);
                    if (szfFile != null)
                    {
                        allSzfFiles.Add(szfFile);
                    }
                }
            }
        }

        return allSzfFiles;
    }

    public async Task<IEnumerable<SzfFile>> GetSzfFilesByTypeAsync(string type)
    {
        var allSzfFiles = await GetAllSzfFilesAsync();
        return allSzfFiles.Where(f => f.Type.Equals(type, StringComparison.OrdinalIgnoreCase)).ToList();
    }

    public async Task DeleteSzfFileAsync(string guid)
    {
        var key = GetKeyForSzfFile(guid);
        await _localStorageService.RemoveItemAsync(key);
    }

    // --- New retrieval methods ---

    public async Task<SzfFile?> GetSzfFileByNameGuidVersionAsync(string name, string guid, string version)
    {
        // For efficiency, first try to retrieve by GUID, as it's a direct key lookup.
        var file = await GetSzfFileByGuidAsync(guid);

        // If found, validate if name and version also match.
        if (file != null &&
            file.Name.Equals(name, StringComparison.OrdinalIgnoreCase) &&
            file.Version.Equals(version, StringComparison.OrdinalIgnoreCase))
        {
            return file;
        }

        return null; // Not found or properties don't match
    }

    public async Task<IEnumerable<SzfFile>> GetSzfFilesByNameAsync(string name)
    {
        var allSzfFiles = await GetAllSzfFilesAsync();
        return allSzfFiles.Where(f => f.Name.Equals(name, StringComparison.OrdinalIgnoreCase)).ToList();
    }

    public async Task<IEnumerable<SzfFile>> GetSzfFilesByNameAndVersionAsync(string name, string version)
    {
        var allSzfFiles = await GetAllSzfFilesAsync();
        return allSzfFiles.Where(f => f.Name.Equals(name, StringComparison.OrdinalIgnoreCase) &&
                                      f.Version.Equals(version, StringComparison.OrdinalIgnoreCase)).ToList();
    }
}